from pydantic import BaseModel, Field
from typing import Optional, Union, List


class Balance(BaseModel):
    currency: Optional[str] = None
    balance: Optional[Union[int, float]] = None


class AppInfo(BaseModel):
    name: Optional[str] = None
    feePercents: Optional[Union[int, float]] = None
    balances: Optional[List[Balance]] = None


class Transfer(BaseModel):
    id: Optional[int] = None
    tgUserId: Optional[int] = None
    currency: Optional[str] = None
    amount: Optional[Union[int, float]] = None
    description: Optional[str] = None


class Withdrawal(BaseModel):
    network: Optional[str] = None
    address: Optional[str] = None
    currency: Optional[str] = None
    amount: Optional[Union[int, float]] = None
    withdrawalId: Optional[str] = None
    status: Optional[str] = None
    comment: Optional[str] = None
    txHash: Optional[str] = None
    txLink: Optional[str] = None


class FeeWithdraw(BaseModel):
    fee: Optional[float] = None
    currency: Optional[str] = None


class Fee(BaseModel):
    networkCode: Optional[str] = None
    feeWithdraw: Optional[FeeWithdraw] = None


class WithdrawalFees(BaseModel):
    code: Optional[str] = None
    minWithdraw: Optional[float] = None
    fees: Optional[List[Fee]] = None


class MultiCheque(BaseModel):
    id: Optional[int] = None
    currency: Optional[str] = None
    total: Optional[int] = None
    perUser: Optional[int] = None
    users: Optional[int] = None
    password: Optional[str] = None
    description: Optional[str] = None
    sendNotifications: Optional[bool] = None
    captchaEnabled: Optional[bool] = None
    refProgramPercents: Optional[int] = None
    refRewardPerUser: Optional[float] = None
    state: Optional[str] = None
    link: Optional[str] = None
    disabledLanguages: Optional[list] = None
    enabledCountries: Optional[list] = None
    forPremium: Optional[int] = None
    forNewUsersOnly: Optional[int] = None
    linkedWallet: Optional[int] = None
    tgResources: Optional[list] = None
    activations: Optional[int] = None
    refRewards: Optional[int] = None


class MultiChequesList(BaseModel):
    total: Optional[int] = None
    limit: Optional[int] = None
    offset: Optional[int] = None
    results: Optional[List[MultiCheque]] = None


class InvoicePayment(BaseModel):
    userId: Optional[int] = None
    paymentNum: Optional[int] = None
    paymentAmount: Optional[int] = None
    comment: Optional[str] = None
    paid: Optional[str] = None


class Invoice(BaseModel):
    id: Optional[str] = None
    amount: Optional[Union[float, int]] = None
    minPayment: Optional[Union[float, int]] = None
    totalActivations: Optional[int] = None
    activationsLeft: Optional[int] = None
    description: Optional[str] = None
    hiddenMessage: Optional[str] = None
    payload: Optional[str] = None
    callbackUrl: Optional[str] = None
    commentsEnabled: Optional[Union[bool, int]] = None
    currency: Optional[str] = None
    created: Optional[str] = None
    paid: Optional[str] = None
    status: Optional[str] = None
    expiredIn: Optional[int] = None
    link: Optional[str] = None
    payments: Optional[List[InvoicePayment]] = []


class InvoicesList(BaseModel):
    total: Optional[int] = None
    limit: Optional[int] = None
    offset: Optional[int] = None
    results: Optional[List[Invoice]] = None


class Currency(BaseModel):
    currency: Optional[str] = None
    name: Optional[str] = None
    minTransfer: Optional[Union[float, int]] = None
    minCheque: Optional[Union[float, int]] = None
    minInvoice: Optional[Union[float, int]] = None
    minWithdraw: Optional[Union[float, int]] = None
    feeWithdraw: Optional[dict] = None


class Subscriptions:
    class Interval(BaseModel):
        interval: Optional[str] = None
        amount: Optional[Union[float, int]] = None
        status: Optional[str] = None
        code: Optional[str] = None

    class tgResource(BaseModel):
        id: Optional[int] = None
        type_: Optional[str] = Field(alias="type", default=None)
        resourceId: Optional[str] = None
        name: Optional[str] = None
        linkedChat: Optional[str] = None


class Subscription(BaseModel):
    id: Optional[int] = None
    name: Optional[str] = None
    description: Optional[str] = None
    currency: Optional[str] = None
    link: Optional[str] = None
    interval: Optional[Subscriptions.Interval] = None
    referralPercent: Optional[int] = None
    returnUrl: Optional[str] = None
    tgResource: Optional[Subscriptions.tgResource] = None


class SubscriptionsList(BaseModel):
    total: Optional[int] = None
    limit: Optional[int] = None
    offset: Optional[int] = None
    results: Optional[List[Subscription]] = None


class SubscriptionCheck(BaseModel):
    subscriptionId: Optional[int] = None
    subscriptionCode: Optional[str] = None
    userId: Optional[int] = None
    amount: Optional[Union[float, int]] = None
    currency: Optional[str] = None
    interval: Optional[str] = None
    refFee: Optional[Union[float, int]] = None
    isRefPay: Optional[bool] = None
    totalAmount: Optional[Union[float, int]] = None
    paymentStart: Optional[str] = None
    paymentEnd: Optional[str] = None
    autoRenewal: Optional[bool] = None
    transactions: Optional[list] = None


class SubscriptionsStatutes:
    ACTIVE = "ACTIVE"
    ARCHIVE = "ARCHIVE"
    DELETED = "DELETED"


class SubscriptionsIntervals:
    DAY = "DAY"
    WEEK = "WEEK"
    MONTH = "MONTH"
    YEAR = "YEAR"
    FOREVER = "FOREVER"


class InvoiceStatutes:
    ACTIVE = "active"
    PAID = "paid"
    EXPIRED = "expired"


class CountriesName:
    DZ: str = "DZ"
    AO: str = "AO"
    BJ: str = "BJ"
    BW: str = "BW"
    BF: str = "BF"
    BI: str = "BI"
    CV: str = "CV"
    CM: str = "CM"
    CF: str = "CF"
    TD: str = "TD"
    KM: str = "KM"
    CG: str = "CG"
    CD: str = "CD"
    DJ: str = "DJ"
    EG: str = "EG"
    GQ: str = "GQ"
    ER: str = "ER"
    SZ: str = "SZ"
    ET: str = "ET"
    GA: str = "GA"
    GM: str = "GM"
    GH: str = "GH"
    GN: str = "GN"
    GW: str = "GW"
    CI: str = "CI"
    KE: str = "KE"
    LS: str = "LS"
    LR: str = "LR"
    LY: str = "LY"
    MG: str = "MG"
    MW: str = "MW"
    ML: str = "ML"
    MR: str = "MR"
    MU: str = "MU"
    MA: str = "MA"
    MZ: str = "MZ"
    NA: str = "NA"
    NE: str = "NE"
    NG: str = "NG"
    RW: str = "RW"
    ST: str = "ST"
    SN: str = "SN"
    SC: str = "SC"
    SL: str = "SL"
    SO: str = "SO"
    ZA: str = "ZA"
    SS: str = "SS"
    SD: str = "SD"
    TZ: str = "TZ"
    TG: str = "TG"
    TN: str = "TN"
    UG: str = "UG"
    ZM: str = "ZM"
    ZW: str = "ZW"
    AF: str = "AF"
    AM: str = "AM"
    AZ: str = "AZ"
    BH: str = "BH"
    BD: str = "BD"
    BT: str = "BT"
    BN: str = "BN"
    MM: str = "MM"
    KH: str = "KH"
    CN: str = "CN"
    CY: str = "CY"
    GE: str = "GE"
    IN: str = "IN"
    ID: str = "ID"
    IR: str = "IR"
    IQ: str = "IQ"
    IL: str = "IL"
    JP: str = "JP"
    JO: str = "JO"
    KZ: str = "KZ"
    KW: str = "KW"
    KG: str = "KG"
    LA: str = "LA"
    LB: str = "LB"
    MY: str = "MY"
    MV: str = "MV"
    MN: str = "MN"
    NP: str = "NP"
    KP: str = "KP"
    KR: str = "KR"
    OM: str = "OM"
    PK: str = "PK"
    PS: str = "PS"
    PH: str = "PH"
    QA: str = "QA"
    SA: str = "SA"
    SG: str = "SG"
    LK: str = "LK"
    SY: str = "SY"
    TW: str = "TW"
    TJ: str = "TJ"
    TH: str = "TH"
    TR: str = "TR"
    TM: str = "TM"
    AE: str = "AE"
    UZ: str = "UZ"
    VN: str = "VN"
    YE: str = "YE"
    AL: str = "AL"
    AD: str = "AD"
    AT: str = "AT"
    BY: str = "BY"
    BE: str = "BE"
    BA: str = "BA"
    BG: str = "BG"
    HR: str = "HR"
    CZ: str = "CZ"
    DK: str = "DK"
    EE: str = "EE"
    FI: str = "FI"
    FR: str = "FR"
    DE: str = "DE"
    GR: str = "GR"
    HU: str = "HU"
    IS: str = "IS"
    IE: str = "IE"
    IT: str = "IT"
    XK: str = "XK"
    LV: str = "LV"
    LI: str = "LI"
    LT: str = "LT"
    LU: str = "LU"
    MT: str = "MT"
    MD: str = "MD"
    MC: str = "MC"
    ME: str = "ME"
    NL: str = "NL"
    MK: str = "MK"
    NO: str = "NO"
    PL: str = "PL"
    PT: str = "PT"
    RO: str = "RO"
    RU: str = "RU"
    SM: str = "SM"
    RS: str = "RS"
    SK: str = "SK"
    SI: str = "SI"
    ES: str = "ES"
    SE: str = "SE"
    CH: str = "CH"
    UA: str = "UA"
    GB: str = "GB"
    VA: str = "VA"
    AG: str = "AG"
    BS: str = "BS"
    BB: str = "BB"
    BZ: str = "BZ"
    CA: str = "CA"
    CR: str = "CR"
    CU: str = "CU"
    DM: str = "DM"
    DO: str = "DO"
    SV: str = "SV"
    GD: str = "GD"
    GT: str = "GT"
    HT: str = "HT"
    HN: str = "HN"
    JM: str = "JM"
    MX: str = "MX"
    NI: str = "NI"
    PA: str = "PA"
    KN: str = "KN"
    LC: str = "LC"
    VC: str = "VC"
    TT: str = "TT"
    US: str = "US"
    AR: str = "AR"
    BO: str = "BO"
    BR: str = "BR"
    CL: str = "CL"
    CO: str = "CO"
    EC: str = "EC"
    GY: str = "GY"
    PY: str = "PY"
    PE: str = "PE" 
    SR: str = "SR"
    UY: str = "UY"
    VE: str = "VE"
    AU: str = "AU"
    FJ: str = "FJ"
    KI: str = "KI"
    MH: str = "MH"
    FM: str = "FM"
    NR: str = "NR"
    NZ: str = "NZ"
    PW: str = "PW"
    PG: str = "PG"
    WS: str = "WS"
    SB: str = "SB"
    TO: str = "TO"
    TV: str = "TV"
    VU: str = "VU"


from ..requests import RequestsClient
from typing import Optional, Union, List
from .models import Balance, Order, Orders, CreateOrder, Currency, WithdrawalCurrency, Store, Withdrawal, Withdrawals, CreateWithdrawal

import json
import hashlib
import time
import hmac


class AsyncFreeKassa(RequestsClient):
    API_HOST: str = "https://freekassa.com/"

    def __init__(self, apiKey: str, shopId: int) -> None:
        """
        Initialize FreeKassa API client
        :param apiKey: Your api key
        :param shopId: Your shop id
        """
        super().__init__()
        self.__apiKey = apiKey
        self.__shopId = shopId
        self.__headers = {
            'Content-Type': 'application/json',
        }
        self.__base_url = "https://api.freekassa.com/v1"
        self.__post_method = "POST"
        self.__payment_name = "freeKassa"
        self.check_values()

    def check_values(self):
        if not self.__shopId or not self.__apiKey:
            raise ValueError('No ShopID or ApiKey specified')

    def __generate_sign(self, data: dict) -> str:
        data = dict(sorted(data.items()))
        return hmac.new(self.__apiKey.encode(), '|'.join(map(str, data.values())).encode(), hashlib.sha256).hexdigest()

    async def get_balance(self) -> List[Balance]:
        """Get balance or your store.
        
        Docs: https://docs.freekassa.com/#operation/getBalance
        """
        url = f"{self.__base_url}/balance"
        
        params = {
            "shopId": self.__shopId,
            "nonce": time.time_ns(),
        }
        params['signature'] = self.__generate_sign(params)
        response = await self._request(self.__payment_name, self.__post_method, url, headers=self.__headers, data=json.dumps(params))
        
        return [Balance(**balance) for balance in response['balance']]
    
    async def get_orders(self, orderId: Optional[int] = None, paymentId: Optional[str] = None, 
                         orderStatus: Optional[int] = None, dateFrom: Optional[str] = None, 
                         dateTo: Optional[str] = None, page: Optional[int] = None) -> Orders:
        """Get orders of your store.
        
        Docs: https://docs.freekassa.com/#operation/getOrders
        
        :param orderId: Optional. Freekassa Order Number. Example: orderId=123456789.
        :param paymentId: Optional. The order number in your store. Example: paymentId=987654321.
        :param orderStatus: Optional. Order status. Example: orderStatus=1.
        :param dateFrom: Optional. Date from. Example: dateFrom=2021-01-01 13:45:21.
        :param dateTo: Optional. Date by. Example: dateTo=2021-01-02 13:45:21.
        :param page: Optional. Page number.
        """
        url = f"{self.__base_url}/orders"
        params = {
            "shopId": self.__shopId,
            "nonce": time.time_ns(),
            "orderId": orderId,
            "paymentId": paymentId,
            "orderStatus": orderStatus,
            "dateFrom": dateFrom,
            "dateTo": dateTo,
            "page": page,
        }
        self._delete_empty_fields(params)
        params['signature'] = self.__generate_sign(params)
        response = await self._request(self.__payment_name, self.__post_method, url, headers=self.__headers, data=json.dumps(params))

        return Orders(pages=int(response['pages']), orders=[Order(**order) for order in response["orders"]])
    
    async def create_order(self, i: int, email: str, ip: str, amount: Union[int, float], currency: str, 
                           tel: Optional[str] = None, paymentId: Optional[str] = None, 
                           successUrl: Optional[str] = None, failureUrl: Optional[str] = None, 
                           notificationUrl: Optional[str] = None) -> CreateOrder:
        """Create an order and receive a payment link.
        
        Docs: https://docs.freekassa.com/#operation/createOrder
        
        :param i: Required. Payment system ID. Example: i=6.
        :param email: Required. Buyer's email address. Example: email=user@site.ru.
        :param ip: Required. Buyer's IP address. Example: ip=0.0.0.0.
        :param amount: Required. Payment amount. Example: amount=100.23.
        :param currency: Required. Payment currency. Example: currency=RUB.
        :param tel: Optional. The payer's phone number is required in some payment methods. Example: tel=+79261231212.
        :param paymentId: Optional. Payment system ID. Example: paymentId=987654321.
        :param successUrl: Optional. Redefining the success url (to enable this parameter, contact support).  Example: successUrl=https://site.ru/success.
        :param failureUrl: Optional. Redefining the error URL (to enable this parameter, contact support). Example: failureUrl=https://site.ru/error.
        :param notificationUrl: Optional. Redefining the notification url (to enable this option, contact support). Example: notificationUrl=https://site.ru/notify.
        """
        url = f"{self.__base_url}/orders/create"
        params = {
            "shopId": self.__shopId,
            "nonce": time.time_ns(),
            "i": i,
            "email": email,
            "ip": ip,
            "amount": amount,
            "currency": currency,
            "tel": tel,
            "paymentId": paymentId,
            "success_url": successUrl,
            "failure_url": failureUrl,
            "notification_url": notificationUrl,
        }
        self._delete_empty_fields(params)
        params['signature'] = self.__generate_sign(params)
        response = await self._request(self.__payment_name, self.__post_method, url, headers=self.__headers, data=json.dumps(params))
        return CreateOrder(**response)
    
    async def get_list_of_currencies(self) -> List[Currency]:
        """Get list of all available currencies for payments.
        
        Docs: https://docs.freekassa.com/#operation/getCurrencies
        """
        url = f"{self.__base_url}/currencies"
        params = {
            "shopId": self.__shopId,
            "nonce": time.time_ns(),
        }
        params["signature"] = self.__generate_sign(params)
        response = await self._request(self.__payment_name, self.__post_method, url, headers=self.__headers, data=json.dumps(params))
        
        return [Currency(**currency) for currency in response['currencies']]
    
    async def check_currency_status(self, paymentId: int) -> bool:
        """Check the availability of the payment system for payment.
        
        Docs: https://docs.freekassa.com/#operation/currencyStatus
        
        :return: True - Payment is available. False - Payment is not available.
        """
        url = f"{self.__base_url}/currencies/{paymentId}/status"
        params = {
            "shopId": self.__shopId,
            "nonce": time.time_ns(),
        }
        params["signature"] = self.__generate_sign(params)
        response = await self._request(self.__payment_name, self.__post_method, url, headers=self.__headers, data=json.dumps(params))
        
        return response['type'] == "success"
  
    async def get_list_of_currencies_for_withdrawal(self) -> List[WithdrawalCurrency]:
        """Get list of all available currencies for withdrawal.
        
        Docs: https://docs.freekassa.com/#operation/getWithdrawalsCurrencies
        """
        url = f"{self.__base_url}/withdrawals/currencies"
        params = {
            "shopId": self.__shopId,
            "nonce": time.time_ns(),
        }
        params["signature"] = self.__generate_sign(params)
        response = await self._request(self.__payment_name, self.__post_method, url, headers=self.__headers, data=json.dumps(params))
        
        return [WithdrawalCurrency(**currency) for currency in response['currencies']]
    
    async def get_list_of_your_stores(self) -> List[Store]:
        """Get list of your stores.
        
        Docs: https://docs.freekassa.com/#operation/getShops
        """
        url = f"{self.__base_url}/shops"
        params = {
            "shopId": self.__shopId,
            "nonce": time.time_ns(),
        }
        params["signature"] = self.__generate_sign(params)
        response = await self._request(self.__payment_name, self.__post_method, url, headers=self.__headers, data=json.dumps(params))
        
        return [Store(**store) for store in response['shops']]
            
    async def get_withdrawals(self, orderId: Optional[int] = None, paymentId: Optional[str] = None, 
                         orderStatus: Optional[int] = None, dateFrom: Optional[str] = None, 
                         dateTo: Optional[str] = None, page: Optional[int] = None) -> Withdrawals:
        """Get list of withdrawals.
        
        Docs: https://docs.freekassa.com/#operation/getWithdrawals
        
        :param orderId: Optional. Freekassa Order Number. Example: orderId=123456789.
        :param paymentId: Optional. The order number in your store. Example: paymentId=987654321.
        :param orderStatus: Optional. Order status. Example: orderStatus=1.
        :param dateFrom: Optional. Date from. Example: dateFrom=2021-01-01 13:45:21.
        :param dateTo: Optional. Date by. Example: dateTo=2021-01-02 13:45:21.
        :param page: Optional. Page number.
        """
        url = f"{self.__base_url}/withdrawals"
        params = {
            "shopId": self.__shopId,
            "nonce": time.time_ns(),
            "orderId": orderId,
            "paymentId": paymentId,
            "orderStatus": orderStatus,
            "dateFrom": dateFrom,
            "dateTo": dateTo,
            "page": page,
        }
        self._delete_empty_fields(params)
        params['signature'] = self.__generate_sign(params)
        response = await self._request(self.__payment_name, self.__post_method, url, headers=self.__headers, data=json.dumps(params))
        
        return Withdrawals(pages=int(response['pages']), withdrawals=[Withdrawal(**withdrawal) for withdrawal in response["orders"]])
    
    async def create_withdrawal(self, i: int, account: str, amount: Union[int, float], currency: str, 
                                paymentId: Optional[str] = None) -> CreateWithdrawal:
        """Create withdrawal.
        
        Docs: https://docs.freekassa.com/#operation/createWithdrawal
        
        :param i: Required. Payment system ID. Example: i=6.
        :param account: Required. A wallet for crediting funds (when paying to FKWallet, withdrawal is made only to your account). Example: account=5500000000000004.
        :param amount: Required. Payment amount. Example: amount=100.23.
        :param currency: Required. Payment currency. Example: currency=RUB.
        :param paymentId: Optional. Payment system ID. Example: paymentId=987654321.
        """
        url = f"{self.__base_url}/withdrawals/create"
        params = {
            "shopId": self.__shopId,
            "nonce": time.time_ns(),
            "i": i,
            "account": account,
            "amount": amount,
            "currency": currency,
            "paymentId": paymentId,
        }
        self._delete_empty_fields(params)
        params['signature'] = self.__generate_sign(params)
        response = await self._request(self.__payment_name, self.__post_method, url, headers=self.__headers, data=json.dumps(params))
        return CreateWithdrawal(**response['data'])
    


from pydantic import BaseModel, Field
from typing import Optional


class CassaInfo(BaseModel):
    id: Optional[int] = None
    name: Optional[str] = None
    status_level: Optional[int] = None
    created_at: Optional[str] = None


class BalanceListField(BaseModel):
    name: Optional[str] = None
    amount: Optional[str] = None
    currency: Optional[str] = None
    amount_accuracy: Optional[int] = None


class CreatePayment(BaseModel):
    id: Optional[str] = None
    url: Optional[str] = None
    type_: Optional[str] = Field(alias="type", default=None)
    rub_amount: Optional[str] = None


class Balance(BaseModel):
    method: Optional[str] = None
    name: Optional[str] = None
    amount: Optional[str] = None
    currency: Optional[str] = None
    amount_accuracy: Optional[int] = None


class PaymentInfo(BaseModel):
    id: Optional[str] = None
    url: Optional[str] = None
    state: Optional[str] = None
    type_: Optional[str] = Field(alias="type", default=None)
    method: Optional[str] = None
    required_method: Optional[str] = None
    amount_currency: Optional[str] = None
    rub_amount: Optional[str] = None
    initial_amount: Optional[str] = None
    remaining_amount: Optional[str] = None
    balance_amount: Optional[str] = None
    commission_amount: Optional[str] = None
    description: Optional[str] = None
    redirect_url: Optional[str] = None
    callback_url: Optional[str] = None
    extra: Optional[str] = None
    created_at: Optional[str] = None
    expired_at: Optional[str] = None
    final_at: Optional[str] = None


class PayoffCreate(BaseModel):
    id: Optional[str] = None
    method: Optional[str] = None
    commission_amount: Optional[str] = None
    amount: Optional[str] = None
    rub_amount: Optional[str] = None
    receive_amount: Optional[str] = None
    deduction_amount: Optional[str] = None
    subtract_from: Optional[str] = None
    amount_currency: Optional[str] = None
    wallet: Optional[str] = None


class PayoffRequest(BaseModel):
    id: Optional[str] = None
    state: Optional[str] = None
    method: Optional[str] = None
    amount: Optional[str] = None
    amount_currency: Optional[str] = None
    commission_amount: Optional[str] = None
    rub_amount: Optional[str] = None
    receive_amount: Optional[str] = None
    deduction_amount: Optional[str] = None
    subtract_from: Optional[str] = None
    wallet: Optional[str] = None
    message: Optional[str] = None
    callback_url: Optional[str] = None
    extra: Optional[str] = None
    created_at: Optional[str] = None
    final_at: Optional[str] = None
    

class SwapPair(BaseModel):
    pair_id: Optional[int] = None
    source: Optional[dict] = None
    target: Optional[dict] = None
    price: Optional[str] = None


class CreateSwap(BaseModel):
    id: Optional[str] = None
    pair_id: Optional[int] = None
    amount_type: Optional[str] = None
    amount: Optional[str] = None
    source: Optional[dict] = None
    target: Optional[dict] = None
    price: Optional[str] = None
    

class SwapInfo(BaseModel):
    id: Optional[str] = None
    state: Optional[str] = None
    pair_id: Optional[int] = None
    amount_type: Optional[str] = None
    amount: Optional[str] = None
    source: Optional[dict] = None
    target: Optional[dict] = None
    price: Optional[str] = None
    created_at: Optional[str] = None
    expired_at: Optional[str] = None
    final_at: Optional[str] = None


class TickersRate(BaseModel):
    base_currency: Optional[str] = None
    currencies: Optional[dict] = None
    
    
class CreateTransfer(BaseModel):
    id: Optional[int] = None
    method: Optional[str] = None
    amount_currency: Optional[str] = None
    amount: Optional[str] = None
    sender: Optional[str] = None
    receiver: Optional[str] = None
    
    
class TransferInfo(BaseModel):
    id: Optional[int] = None
    state: Optional[str] = None
    type_: Optional[str] = Field(alias="type", default=None)
    method: Optional[str] = None
    amount_currency: Optional[str] = None
    amount: Optional[str] = None
    sender: Optional[str] = None
    receiver: Optional[str] = None
    description: Optional[str] = None
    created_at: Optional[str] = None
    final_at: Optional[str] = None
    
    
class Stats(BaseModel):
    payed_rub_amount: Optional[str] = None
    payed_count: Optional[int] = None
    total_count: Optional[int] = None
    conversion_percent: Optional[int] = None
    


from pydantic import BaseModel
from typing import Union, Optional


class Balance(BaseModel):
    balance: Optional[float] = None
    referral: Optional[float] = None
    hold: Optional[float] = None

class Order(BaseModel):
    id: Optional[str] = None
    order_id: Optional[Union[int, str]] = None
    desc: Optional[str] = None
    merchant_id: Optional[str] = None
    merchant_domain: Optional[str] = None
    method: Optional[str] = None
    amount: Optional[Union[int, float]] = None
    currency: Optional[str] = None
    profit: Optional[float] = None
    commission: Optional[float] = None
    commission_client: Optional[float] = None
    commission_type: Optional[str] = None
    email: Optional[str] = None
    status: Optional[str] = None
    date: Optional[str] = None
    expired_date: Optional[str] = None
    complete_date: Optional[str] = None
    us_vars: Optional[Union[dict, str, list]] = None

class OrderMethodCurrencies(BaseModel):
    RUB: Optional[float] = None
    UAH: Optional[float] = None
    USD: Optional[float] = None
    EUR: Optional[float] = None

class OrderMethod(BaseModel):
    name: Optional[str] = None
    min: Optional[OrderMethodCurrencies] = None
    max: Optional[OrderMethodCurrencies] = None
    commission_percent: Optional[float] = None

class Withdrawal(BaseModel):
    id: Optional[str] = None
    my_id: Optional[Union[int, str]] = None
    method: Optional[str] = None
    wallet: Optional[str] = None
    amount: Optional[float] = None
    amount_down: Optional[float] = None
    commission: Optional[float] = None
    commission_type: Optional[int] = None
    status: Optional[str] = None
    cancel_message: Optional[str] = None
    date: Optional[str] = None
    complete_date: Optional[str] = None

class WithdrawalMethod(BaseModel):
    name: Optional[str] = None
    min: Optional[float] = None
    max: Optional[float] = None
    commission_percent:Optional[float] = None
    commission_sum: Optional[float] = None

class CreateWithdrawalInfo(BaseModel):
    id: Optional[str] = None
    my_id: Optional[Union[str, int]] = None
    method: Optional[str] = None
    wallet: Optional[str] = None
    amount: Optional[float] = None
    amount_in_currency: Optional[float] = None
    amount_currency: Optional[str] = None
    amount_rate: Optional[float] = None
    amount_down: Optional[float] = None
    commission: Optional[float] = None
    commission_type: Optional[int] = None
    status: Optional[str] = None